
; ***********************************************************************************************
;	Port-Definitionen			(c) 2024 by DC7GB unter: BY-NC-SA 4.0
; ***********************************************************************************************


; -------
; Port B:
; -------

.equ	DRB		= 0b101101
.equ	PULUPB		= 0b000101
.equ	SCL		= PB0		; (5) OUT: (MOSI) SCL fr SI514
.equ	RXPILOT		= PB1		; (6) INP: (MISO) 
.equ	SDA		= PB2		; (7) OUT: (SCK) SDA fr SI541
.equ	TEST		= PB3		; (2) OUT: Testausgang
.equ	INPUT		= PB4		; (3) ANA: ADC2
.equ	RESET		= PB5		; (1) INP: *** unbenutzt ***



; ***********************************************************************************************
;	System-Konstantendefinitionen
; ***********************************************************************************************

;---------------------------------------
; Systemflags 0

.equ	TIM10MS		= 0		; 0= Timer luft / 1= Timer ist abgelaufen
.equ	SI514FOUND	= 1		; 0= SI514 nicht gefunden / 1= SI514 meldet sich
.equ	SI514ERR	= 2		; 0= kein Fehler / 1= Zugriffsfehler
.equ	SI541UPDATE	= 3		; 0= kein Update / 1= Parameter in SI514 schreiben
.equ	ACK		= 4		; 0= keine Antwort / 1= I2C-Baustain hat geantwortet
.equ	F0_5		= 5		; 0=
.equ	F0_6		= 6		; 0=
.equ	F0_7		= 7		; 0= 

; -------------------------------------
; Bit-Codes

.equ	BIT0		= 0
.equ	BIT1		= 1
.equ	BIT2		= 2
.equ	BIT3		= 3
.equ	BIT4		= 4
.equ	BIT5		= 5
.equ	BIT6		= 6
.equ	BIT7		= 7

; -------------------------------------
; Zeiten und Zhler

.equ	TAKT		= 8000000			; Tasktfrequenz in Hz
.equ	SYSTAKT		= 100				; 10ms Timer-Rate
.equ	TICS0		= (TAKT/1024)/SYSTAKT-1		; = 77 mit Vorteiler 1024

; Intervalle in 10ms-Tics:

.equ	DELAY190MS	= 10		; 100ms Buserror-Timeout
.equ	DELAY300MS	= 30		; 300ms Break-Timeout

; Si514-Parameter:

.equ	SI514W		= ($55 << 1)	; I2C-Adresse des Oszillators mit Write in Bit0
.equ	VCOMIN		= 2080000	; minimale VCO-Frequenz in kHz
.equ	VCOMAX		= 2500000	; maximale VCO-Frequenz in kHz
.equ	FOUTMIN		= 100		; minimale Ausgangsfrequent in kHz
.equ	FOUTMAX		= 250000	; maximale Ausgangsfrequenz in kHz
.equ	HSDIVMIN	= 10		; Minimalwert fr HS_DIV
.equ	HSDIVMAX	= 1022		; Maximalwert fr HS_DIV
.equ	FSI514QUARZ	= 31980		; Quarzfrequenz in kHz
.equ	ZWEIHOCH29	= (1 << 29)	; Frac-Multiplikator (32 Bit)

; sonstiges Konstanten:

.equ	SPACE		= $20		; ASCII-Abstand
.equ	I2CWDH		= 4		; Anzahl der I2C-Zugriffsversuche


; ***********************************************************************************************
;	System-Variablendefinition
; ***********************************************************************************************

; Variablen in CPU-Registern
; --------------------------

; ----------------------------------------------
; Register:

.def	akku0		= r0		; Artihmetik-
.def	akku1		= r1		; ...Buffer
.def	akku2		= r2		; ...fr
.def	akku3		= r3		; ...Rechnungen
.def	akku4		= r4		; ...und
.def	akku5		= r5		; ...Zwischenergebnisse
.def	akku6		= r6		; ...mit
.def	akku7		= r7		; ...64Bit Genauigkeit
;
.def	rest		= r14		; Divisionsrest beim Teilen
.def	const		= r15		; Konstantenspeicher
.def	mkand		= r16		; Multiplikand
.def	tics		= r17		; durchlaufender 10ms-Zhler
;
.def	temp		= r18		; *** allgemeines Hilfsregister
.def	counter		= r19		; *** allgemeiner Zhler
.def	buffer		= r20		; *** allgemeiner Buffer
;
.def	flags0		= r21		; Systemflags 1

; ----------------------------------
; Ab r26 Doppelbelegung mit X, Y, Z
; ----------------------------------

