.LIST
.LISTMAC
.include	"macros.asm"
.include	"def.asm"
; *******************************************************************************************
;	Oszillator fr ATtiny85						 (c) 2024 by DC7GB
; *******************************************************************************************
;
;	Copyright-Vermerk
;	=================
;	Alle Routinen und Include-Dateien (sofern nicht anders angegeben) lizenziert unter:
;	Creative Commons BY-NC-SA 4.0 (https://creativecommons.org/licenses/by-nc-sa/4.0/)
;	Konstanten des Si514 sind dem Datenblatt von Skyworks Rev. 1.2 vom 12. August 2021
;	entnommen. Rechnung berprft mit: ProgOsc Calculator 19.3.0 (54924) fr Windows 10
;	von Silicon Laboratories.
;
;	Bei Weitergabe, alle nderungen im Code als Kommentar markieren und in main.asm
;	zusammen gefasst in der Versionshistorie beschreiben.
;
;
;	========
;	ACHTUNG: Beim Programmieren diese Fuse-Bits berprfen/einstellen: $ff $df $e2
;	========
;
;
;	Frequenzeingabe:
;	================
;	Die Frequenz muss in kHz als ASCII-Code von EEPROM-Adresse $1...$6 linksbndig
;	eingegeben werden. Abschluss mit SPACE, Nullbyte oder nach max. 6 ASCII-Zeichen.
;	EEPROM-Beispiel fr 1.197 MHz:
;
;		:06 000000 00313139370028 -->	0. Position = $00 (Parkadresse fr EEPROM)
;						1. Position = $31 (= ASCII 1)
;						2. Position = $31 (= ASCII 1)
;						3. Position = $39 (= ASCII 9)
;						4. Position = $37 (= ASCII 7)
;						5. Position = $00 (= Ende Frequenzeingabe)
;
;	Die Zeile beginnt mit der Anzahl der Daten (hier $06) und einer 24Bit-Arresse
;	(hier $000000) mit der Adresse des ersten Datenbytes im Rahmen. Am Rahmenende
;	steht eine Frfsumme ($28). Alle anderen Bytes in in nachfolgenden Rahmen haben
;	fr das Programm keine Bedeutung.
;
;     *	Frequenzbereich: 100 kHz bis 250 MHz in 1kHz-Schritten.
;
;
;	Fehlerausgabe:
;	==============
;	Fehler werden als wiederholte Impulsfolge am Ausgang TEST ausgegeben. Die Anzahl
;	der H-Pulse codiert die Fehlernummer. Pulsdauer = 10ms, Pulspause = 10ms, Abstand
;	zwischen den Fehlerausgaben = 200ms. Der Code kann z.B. mit einem Oszilloskop an
;	TEST angezeigt werden. Folgende Fehler sind mglich:
;
;		Anzahl H-Pulse		Fehlerbeschreibung
;		-------------------------------------------------------------------------
;			1		ungltiges ASCII-Zeichen in ASCII-Frequenz
;			2		ASCII-Frequenz kleiner als 100kHz
;			3		ASCII-Frequenz grer als 250MHz
;			4		LSDIV ausserhalb des erlaubten Bereichs 1..32
;			5		HSDIV ausserhalb des erlaubten Bereichs 10..1022
;			6		I2C-Fehler: Keine Antwort vom Si514, Adressfehler?



; ***********************************************************************************************
;	Programmstart
; ***********************************************************************************************
.dseg
.org		SRAM_START			; RAM-Anfang setzen


memstart:

.cseg
.org		0

		rjmp	START			; Reset Handler 
EXT_INT0:	rjmp	EXT_INT0		; IRQ0-Handler
PCINT0_V:	rjmp	PCINT0_V		; PCINT0-Handler
TIM1_COMPA:	rjmp	TIM1_COMPA		; Timer1 Compare-A Handler
TIM1_OVF:	rjmp	TIM1_OVF		; Timer1 Overflow Handler
TIM0_OVF:	rjmp	TIM0_OVF		; Timer0 Overflow Handler
EE_RDY:		rjmp	EE_RDY			; EEPROM-Handler
ANA_COMP:	rjmp	ANA_COMP		; Analog Comparator Handler
ADC_V:		rjmp	ADC_V			; ADC Ready Handler
TIM1_COMPB:	rjmp	TIM1_COMPB		; Timer1 Compare-B Handler
		rjmp	TIM0_COMPA		; Timer0 Compare-A Handler
TIM0_COMPB:	rjmp	TIM0_COMPB		; Timer0 Compare-B Handler
WDT:		rjmp	WDT			; Watchdog Handler
USI_START:	rjmp	USI_START		; USI-Handler
USI_OVF:	rjmp	USI_OVF			; USI Overlflow Handler



; ----------------------------------------------
; Einsprungpunkt nach RESET

START:		clt				;; Langstart setzen
RESTART:	cli				;; sicherheitshalber alle Interrupt auch manuell sperren
		ldi	temp,low(RAMEND)	;; lower 8 Bit
		out	SPL,temp		;; ...des SP laden
		ldi	temp,high(RAMEND)	;; higher 8 Bit
		out	SPH,temp		;; ...des SP laden

; ----------------------------------------------
; Speicher lschen:

		clr	temp
		ldiw	Z,(memstart)
INIT1:		st	Z+,temp			;; RAM-Zellen ab (memstart) lschen
		cpiw	Z,(RAMEND+1)		;; RAM-Ende erreicht?
		brne	init1			;; nein...
;
		rcall	init			;; System initialisieren


; ***********************************************************************************************
; Beginn des Hauptprogramms			(in dieser Version ohne Funktion)
; ***********************************************************************************************

	
WAIT:		sei				; Interrupt frei geben
		sleep				; warte auf nchsten Interrupt
		sbrs	flags0,TIM10MS		; 10ms Timer abgelaufen?
		rjmp	wait			; nein...
; ----------------------------------------------
		cbr	flags0,(1<<TIM10MS)	; warte auf nchstes Timer-Ereignis

	; KEINE AKTION IN DIESER VERSION

		rjmp	wait			; endlos scheifen...


; ----------------------------------------------------------------------------------------------

.include	"data.asm"
.include	"init.asm"		
.include	"subroutinen.asm"
.include	"si514.asm"
.include	"interrupt.asm"
