.cseg
; ***********************************************************************************************
;	Subroutinen fr den Si514		(c) 2024 by DC7GB unter: BY-NC-SA 4.0
; ***********************************************************************************************


; -----------------------------------------------------------------------------------------------
; (Reg 128)	RESET Si514
; -----------------------------------------------------------------------------------------------

RESETSI514:	ldi	temp,128		; Register #
		sts	regadr,temp		; ...speichern
		ldi	temp,2			; Anzahl der zu sendenden Daten
		sts	i2canzahl,temp		; ...(regadr und i2cdaten=reg132)
;
		ldi	temp,(1<<BIT7)		; setze
		sts	reg128,temp		; ...RST-Bit
		rcall	i2cout			; Daten ausgeben (gibt CY=1 zurck, kein Fehler!)
;
		rcall	wait200ms		; warte auf Beruhigung
; -------------
		ret



; -----------------------------------------------------------------------------------------------
; Sende berechnete Parameter zum Si514, starte Kalibrierung:
; -----------------------------------------------------------------------------------------------

SEND2SI514:

; ----------------------------------------------
; (Reg 132)	Schalte Output hochohmig:

SEND1:		ldi	temp,132		; Register #
		sts	regadr,temp		; ...speichern
		ldi	temp,2			; Anzahl der zu sendenden Daten
		sts	i2canzahl,temp		; ...(regadr und i2cdaten=reg132)
;
		clr	temp			; lscche
		sts	reg132,temp		; ...OE-Bit
		rcall	i2cout			; Daten ausgeben
		brcc	send2			; --> I2C-Sendung ok
SENDHALT:	ldi	temp,6			; Fehler #6
		rcall	error			; ...als Impuls anzeigen
		rjmp	sendhalt

; ----------------------------------------------
; (Reg 0)	bertrage L1 und L2

SEND2:		ldi	temp,0			; erste Register #
		sts	regadr,temp		; ...speichern
		ldi	temp,2			; Anzahl der zu sendenden Daten
		sts	i2canzahl,temp		; ...(regadr und i2cdaten=reg0)
;
		lds	temp,lp1
		swap	temp
		lds	buffer,lp2
		or	temp,buffer		; (LP1:LP2)
		sts	reg0,temp		; ...bergeben
		rcall	i2cout			; Daten ausgeben
		brcc	send3			; --> I2C-Sendung ok
		rjmp	sendhalt

; ----------------------------------------------
; (Reg 5..11)	PLL-Daten

SEND3:		ldi	temp,5			; erste Register #
		sts	regadr,temp		; ...speichern
		ldi	temp,8			; Anzahl der zu sendenden Daten
		sts	i2canzahl,temp		; ...(regadr und i2cdaten=reg5..11)
;
		ldiw	Z,m			; Quelle:
		ldiw	Y,reg5			; Ziel:
		ldi	counter,5		; m-Buffer fr
		rcall	movez2y			; ...Si514 kopieren
		lds	temp,hsdiv		; HSDIV (L)
		st	Y+,temp			; ...in Reg10 speichern
		lds	temp,(hsdiv+1)		; HSDIV (H) laden
		lds	buffer,lsdivcode	; Si514-Code fr LSDIV in
		swap	buffer			; ...H-Nibble bringen
		or	temp,buffer		; ...mit HSDIV (H) vereinen und
		st	Y,temp			; ...in Reg11 speichern
		rcall	i2cout			; Daten ausgeben
		brcc	send4			; --> I2C-Sendung ok
SENDERR:	rjmp	sendhalt

; ----------------------------------------------
; (Reg 132)	FCAL

SEND4:		ldi	temp,132		; Register #
		sts	regadr,temp		; ...speichern
		ldi	temp,2			; Anzahl der zu sendenden Daten
		sts	i2canzahl,temp		; ...(regadr und i2cdaten=reg132)
;
		ldi	temp,1			; setze
		sts	reg132,temp		; ...CAL-Bit
		rcall	i2cout			; Daten ausgeben	
		brcs	senderr			; keine Antwort vom Si514...
;
		rcall	wait20ms		; Kalibrierung, etwas warten

; ----------------------------------------------
; (Reg 132)	aktiviere Output:

SEND5:		ldi	temp,132		; Register #
		sts	regadr,temp		; ...speichern
		ldi	temp,2			; Anzahl der zu sendenden Daten
		sts	i2canzahl,temp		; ...(regadr und i2cdaten=reg132)
;
		ldi	temp,(1<<BIT2)		; setze
		sts	reg132,temp		; ...OE-Bit
		rcall	i2cout			; Daten ausgeben
		brcc	sendsi514_end		; --> I2C-Sendung ok
		rjmp	sendhalt
; -------------
SENDSI514_END:

	;rcall	wait200ms
	;rjmp	SEND2SI514

		ret



; -----------------------------------------------------------------------------------------------
; Parameter fr Si514 aus der binren Frequenz ausrechnen:		(ca. 2,5ms @ 8MHz)
; -----------------------------------------------------------------------------------------------

SI514QRG:

; ---------------------------------------------
; LS_DIV berechnen:
; ---------------------------------------------

		rcall	quotient		; berechne VCOMIN / FOUT
; / HSDIVMAX
		ldi	temp,low(HSDIVMAX)	; lade die
		mov	akku0,temp		; ...maximale
		ldi	temp,high(HSDIVMAX)	; ...Konstante HSDIVMAX
		mov	akku1,temp		; ...als Divisor
		ldiw	Z,buf64			; Ergebnisbuffer
		rcall	div64z32		; --> VCOMIN / (FOUT * HSDIVMAX)
; LSDIV < 1?
		ldiw	Z,buf64			; lade Ergebnisbuffer
		ldd	temp,Z+4		; ist
		ldd	buffer,Z+5		; ...das
		or	temp,buffer		; ...Ergebnis
		ldd	buffer,Z+6		; ...von
		or	temp,buffer		; ... VCOMIN /
		ldd	buffer,Z+7		; ...(FOUT * HSDIVMAX)
		or	temp,buffer		; ...kleiner als 1?
		brne	siq_lsdivganz		; nein...
		ldi	temp,1			; setze LSDIV auf 1
		rjmp	siq_lsdivteil
; ganzzahlig?
SIQ_LSDIVGANZ:	ldd	temp,Z+4		; Lade ganzzahligen Anteil von LSDIV
		ld	counter,Z		; gibt
		ldd	buffer,Z+1		; ...es
		or	counter,buffer		; ...einen
		ldd	buffer,Z+2		; ...gebrochenen
		or	counter,buffer		; ...Anteil
		ldd	buffer,Z+3		; ...von
		or	counter,buffer		; ...LSDIV?
		breq	siq_lsdivteil		; nein...
		lsl	temp			; nchst mglichen LSDIV whlen
; LSDIV-Teiler:
SIQ_LSDIVTEIL:	clr	counter			; LSDIV: 0..5 = Teiler 1,2,4,8,16 oder 32
		ldi	buffer,1		; minimaler LSDIV-Teiler
SIQ_LOOP2:	cp	temp,buffer		; Teilerwert exakt erreicht?
		breq	siq_lsdivok		; ja...
		brcs	siq_lsdivok		; gerade berschritten...
		inc	counter			; nchsten LSDIV-Code
		lsl	buffer			; ...und nchsten Teiler einstellen
		cpi	buffer,64		; Ende erreicht?
		brne	siq_loop2
SIQ_WAIT3:	ldi	temp,4			; ..> Fehler #4
		rcall	error			; ...als Impuls anzeigen
		rjmp	siq_wait3
; korr:
SIQ_LSDIVOK:	sts	lsdivcode,counter	; LS_DIV-Code fr Si514
		sts	lsdiv,buffer		; ...und gerundeten Teiler speichern
; =============
;	rjmp	test_lsdiv		; Anzeige bei 1197 kHz:	LSDIV=$02
; =============

; ---------------------------------------------
; HS_DIV berechnen:
; ---------------------------------------------

		rcall	quotient		; berechne VCOMIN / FOUT
		lds	temp,lsdiv		; durch 1
		cpi	temp,1			; ...teilen?
		breq	siq_hsdivchk1		; ja...
		lds	akku0,lsdiv		; lade LS_DIV
		clr	akku1			; ...mit
		clr	akku2			; ...32Bit
		clr	akku3			; ...Erweiterung
		ldiw	Z,buf64			; Ergebnisbuffer
		rcall	div64z32		; --> VCOMIN / (FOUT * LS_DIV)
; ganzzahlig?
SIQ_HSDIVCHK1:	ldd	YL,Z+4			; ganzzahliges
		ldd	YH,Z+5			; ...Ergebnis
		ld	counter,Z		; gibt
		ldd	buffer,Z+1		; ...es
		or	counter,buffer		; ...einen
		ldd	buffer,Z+2		; ...gebrochenen
		or	counter,buffer		; ...Anteil
		ldd	buffer,Z+3		; ...von
		or	counter,buffer		; ...HSDIV?
		breq	siq_hsdivchk2		; nein...
		adiw	Y,1			; nchst hheren Wert einstellen
; gerade?
SIQ_HSDIVCHK2:	sbrc	YL,BIT0			; HSDIV gerade?
		adiw	Y,1			; nchst hheren Wert einstellen
		stsw	hsdiv,Y			; speichere 16Bit HSDIV
; =============
;	rjmp	test_hsdiv		; Anzeige bei 1197 kHz:	HSDIV=$0366
; =============

; ---------------------------------------------
; Teste Bereichsberschreitung von HSDIV:

		lds	YL,hsdiv		; lade 16Bit
		lds	YH,(hsdiv+1)		; ...Ergebnis
		cpiw	Y,HSDIVMAX		; Maximum berschritten?
		brcs	siq_vco			; nein...
SIQ_WAIT4:	ldi	temp,5			; ..> Fehler #5
		rcall	error			; ...als Impuls anzeigen
		rjmp	siq_wait4

; ---------------------------------------------
; Kontrolle von VCOMIN, ggf. HSDIV+2:

SIQ_VCO:	lds	temp,hsdiv		; lade
		st	Z,temp			; ...16
		lds	temp,(hsdiv+1)		; ...Bit
		std	Z+1,temp		; ...HSDIV
		clr	temp			; ...mit
		std	Z+2,temp		; ...32Bit
		std	Z+3,temp		; ...Erweitrung
		lds	temp,lsdiv		; ...erneut
		rcall	mul64z8			; --> HSDIV * LSDIV
;
		lds	akku0,binqrg		; Oszillatorfrequnz
		lds	akku1,(binqrg+1)	; ...FOUT laden
		lds	akku2,(binqrg+2)	; ...als 32Bit
		lds	akku3,(binqrg+3)	; ...Wert laden
		rcall	mul64z32		; --> (HSDIV * LSDIV) * FOUT
; -------------
;	rjmp	test_buf32		; Anzeige bei 1197 kHz:	VCO = $001fc7dc
; -------------
		ld	temp,Z			; erreicht
		ldi	buffer,low(VCOMIN)	; ...der neu
		cp	temp,buffer		; ...berechnete
		ldd	temp,Z+1		; ...Wert
		ldi	buffer,byte2(VCOMIN)	; ...von
		cpc	temp,buffer		; ...(HSVIV * LSDIV * FOUT)
		ldd	temp,Z+2		; ...das
		ldi	buffer,byte3(VCOMIN)	; ...Minimum
		cpc	temp,buffer		; ...der
		ldd	temp,Z+3		; ...erlaubten
		ldi	buffer,byte4(VCOMIN)	; ...VCO-
		cpc	temp,buffer		; ...Frequenz?
		brcc	siq_m			; ja...
; HSDIV + 2:
		ldsw	Y,hsdiv			; lade 16Bit HSDIV
		adiw	Y,2			; ...erhhe um 2
		stsw	hsdiv,Y			; speichere korrigiertes HSDIV
		rjmp	siq_vco			; teste erneut...
		
; ---------------------------------------------
; M_INT und FRAC berechnen:
; ---------------------------------------------

SIQ_M:		rcall	clrbuf64		; lsche Rechenbuffer
		ldiw	Z,buf64			; lade Rechenbuffer
		lds	temp,hsdiv		; lade
		st	Z,temp			; ...HS_DIV(L)
		lds	temp,(hsdiv+1)		; lade
		std	Z+1,temp		; ...HS_DIV_H
		clr	const			; lsche
		std	Z+2,const		; ...32Bit Erweiterung
		std	Z+3,const		; ...von HSDIV
		lds	akku0,binqrg		; FOUT
		lds	akku1,(binqrg+1)	; ...als
		lds	akku2,(binqrg+2)	; ...32Bit Wert
		lds	akku3,(binqrg+3)	; ...laden
		rcall	mul64z32		; --> HS_DIV * FOUT
		lds	temp,lsdiv		; lade LS_DIV
		rcall	mul64z8			; --> (HS_DIV * FOUT) * LS_DIV
; -------------
;	rjmp	test_buf32		; Anzeige bei 1197 kHz:	VCO = $001fc7dc
; -------------
		ldi	temp,low(ZWEIHOCH29)	; Lade
		mov	akku0,temp		; ...die
		ldi	temp,byte2(ZWEIHOCH29)	; ...Verschiebe-
		mov	akku1,temp		; ...konstante
		ldi	temp,byte3(ZWEIHOCH29)	; ...2**29
		mov	akku2,temp		; ...zur
		ldi	temp,byte4(ZWEIHOCH29)	; ...FRAC-
		mov	akku3,temp		; ...Berechnung
		rcall	mul64z32		; --> (HS_DIV * FOUT * LS_DIV) * 2**29
; -------------
;	rjmp	test_buf64		; Anzeige bei 1197 kHz:	ZSHIFT29=$0003f8fb80000000
; -------------
		ldi	temp,low(FSI514QUARZ)	; lade
		mov	akku0,temp		; ...die
		ldi	temp,byte2(FSI514QUARZ)	; ...interne
		mov	akku1,temp		; ...Quarz-
		ldi	temp,byte3(FSI514QUARZ)	; ...frequenz
		mov	akku2,temp		; ...der
		ldi	temp,byte4(FSI514QUARZ)	; ...Si514
		mov	akku3,temp		; ...PLL
		rcall	div64z32		; --> (HS_DIV * FOUT * LS_DIV * 2**29) / FSI514QUARZ
; =============
;	rjmp	test_buf64		; Anzeige bei 1197 kHz:	FRAC=$0000000824152219
; =============
		ldd	akku0,Z+3		; lade higher
		ldd	akku1,Z+4		; ...Ergebnisbits
		ldi	counter,3		; wegen 2**29
SIQ_LOOP4:	lsl	akku0			; schiebe ganzzahligen
		rol	akku1			; ...Anteil nach akku1
		dec	counter			; fertig?
		brne	siq_loop4		; nein...
		sts	mint,akku1		; speichere MINT
; =============
;	rjmp	test_mint		; Anzeige bei 1197 kHz:	MINT=$41
; =============

; ---------------------------------------------
; Ergebnis M retten:

		ldiw	Z,buf64			; Quelle:
		ldiw	Y,m			; Ziel:
		ldi	counter,5		; m-Buffer
		rcall	movez2y			; Daten kopieren

; ---------------------------------------------
; LP1 und LP2 ber FVCO berechnen:
; ---------------------------------------------

		rcall	clrbuf64		; lsche Rechenbuffer
		lds	akku0,binqrg		; Oszillatorfrequnz
		lds	akku1,(binqrg+1)	; ...FOUT laden
		lds	akku2,(binqrg+2)	; ....als 32Bit
		lds	akku3,(binqrg+3)	; ...laden
		ldiw	Z,buf64			; lade Rechenregister
		lds	temp,lsdiv		; LSDIV
		st	Z,temp			; ...speichern
		rcall	mul64z32		; --> FOUT * LSDIV
		lds	akku0,hsdiv		; HSDIV
		lds	akku1,(hsdiv+1)		; ...speichern
		clr	akku2			; ...und higher 32Bit
		clr	akku3			; ...lschen
		rcall	mul64z32		; --> (FOUT * LSDIV) * HSDIV
; =============
;	rjmp	test_buf32		; Anzeige bei 1197 kHz:	$001fc7dc
; =============
		ldiw	Z,buf64			; speichere
		ldiw	Y,fvco			; ...das
		ldi	counter,4		; ...Ergebnis
		rcall	movez2y			; ...als FVCO
; suche LP1,LP2
		lds	akku0,buf64		; lade
		lds	akku1,(buf64+1)		; ...berechnete
		lds	akku2,(buf64+2)		; ...VCO-Frequenz
		lds	akku3,(buf64+3)		; ...bei FOUT und Si514-Parametern
		ldiw	Z,(2*LPTABELLE)		; Tabelle mit LP-Konstanten
		ldi	counter,5		; teste 5 Eintrge
SIQ_LOOP5:	lpm	akku4,Z+		; lade
		lpm	akku5,Z+		; ...Vergleichswert
		lpm	akku6,Z+		; ...aus der
		lpm	akku7,Z+		; ...Tabelle
		cp	akku0,akku4		; ist
		cpc	akku1,akku5		; ...der
		cpc	akku2,akku6		; ...berechnete
		cpc	akku3,akku7		; ...FVCO < Tabellenwert
		brcs	siq_fvcolower		; ja...
		dec	counter			; fertig?
		breq	siq_fvcolower		; ja, verwende Maximwerte...
		adiw	Z,2			; zeige auf nchsten Eintrag
		rjmp	siq_loop5
; found:
SIQ_FVCOLOWER:	lpm	temp,Z+			; lade LP1
		sts	lp1,temp
		lpm	buffer,Z+		; lade LP2
		sts	lp2,buffer
; =============
;	rjmp	test_lp		; Anzeige bei 1197 kHz:	$0201
; =============
		ret



; -----------------------------------------------------------------------------------------------
; Berechne VCOMIN / FOUT im Format F32.32 in (buf64):
; -----------------------------------------------------------------------------------------------

QUOTIENT:	lds	akku0,binqrg		; Oszillatorfrequnz
		lds	akku1,(binqrg+1)	; ...FOUT laden
		lds	akku2,(binqrg+2)	; ...als 32Bit
		lds	akku3,(binqrg+3)	; ...Wert laden
;
		rcall	clrbuf64		; Lsche Rechenbuffer
		ldiw	Z,buf64			; Ergebnisbuffer
		ldi	temp,low(VCOMIN)	; lade
		std	Z+4,temp		; ...die
		ldi	temp,byte2(VCOMIN)	; ...minimale
		std	Z+5,temp		; ...VCO-
		ldi	temp,byte3(VCOMIN)	; ...Frequenz *2^32
		std	Z+6,temp		; ...als
		ldi	temp,byte4(VCOMIN)	; ...64-Bit
		std	Z+7,temp		; ...Dividend
		rcall	div64z32		; --> VCOMIN / FOUT
; -------------
		ret



; -----------------------------------------------------------------------------------------------
; Konvertiere die ASCII-Frequenz in 32Bit HEX (in 1 kHz-Auflsung)	(ca. 1,5s @ 8MHz)
; -----------------------------------------------------------------------------------------------

ASCQRG2BIN:	rcall	clrbuf64		; Lsche den Ergebnisbuffer
		ldiw	Z,ascadr		; hchste ASCII-Stelle
		ldi	counter,6		; maximal 6 ASCII-Stellen bercksichtigen
; conv:
A2B_CONV:	ld	temp,Z+			; lade Ziffer
		tst	temp			; Ende gefunden?
		breq	ascqrg2bin_chk		; ja...
		brmi	ascqrg2bin_chk		; ja...
		cpi	temp,SPACE		; Ende gefunden?
		breq	ascqrg2bin_chk		; ja...
		cpi	temp,'0'		; Zifferntest1 ok?
		brcs	a2b_wait1		; nein...
		cpi	temp,'9'+1		; Zifferntest2 ok?
		brcc	a2b_wait1		; nein...
;
		push	ZL			; rette
		push	ZH			; ...ASCII-Adresse
		subi	temp,'0'		; konvertiere ASC nach BIN
		push	temp			; ...und rette das binre Ergebnis
		ldiw	Z,buf64			; Lade Ergebnisbuffer
		ldi	temp,10			; verschiebe binres Ergebnis
		rcall	mul64z8			; ...um eine Dezimalstelle nach links
; Inp + buf64:
		pop	akku0			; hole Bin-Stelle zurck
		clr	const			; 0-Konstante
		ldiw	Z,buf64			; Adresse des Ergebnisbuffers
		ld	temp,Z			; lade 1. Ergebnisstelle
		add	temp,akku0		; ...und addiere
		st	Z,temp			; ...binren
		ldd	temp,Z+1		; ...Stellenwert
		adc	temp,const		; ...zum
		std	Z+1,temp		; ...32Bit
		ldd	temp,Z+2		; ...Ergebnis
		adc	temp,const		; ...der
		std	Z+2,temp		; ...dezimalen
		ldd	temp,Z+3		; ...Frequenz-
		adc	temp,const		; ...eingabe
		std	Z+3,temp
		pop	ZH			; lade ASCII-Adresse
		pop	ZL			; ...zurck
; 
		dec	counter			; fertig?
		brne	a2b_conv		; nein...
; -------------
ASCQRG2BIN_CHK:	ldiw	Z,buf64			; Adresse des Ergebnisbuffers
		ld	temp,Z
		cpi	temp,low(FOUTMIN)
		ldd	temp,Z+1
		ldi	buffer,byte2(FOUTMIN)
		cpc	temp,buffer
		ldd	temp,Z+2
		ldi	buffer,byte3(FOUTMIN)
		cpc	temp,buffer
		ldd	temp,Z+3
		ldi	buffer,byte4(FOUTMIN)
		cpc	temp,buffer
		brcs	a2b_wait2		; FOUT zu kein...
;
		ld	temp,Z
		cpi	temp,low(FOUTMAX+1)
		ldd	temp,Z+1
		ldi	buffer,byte2(FOUTMAX+1)
		cpc	temp,buffer
		ldd	temp,Z+2
		ldi	buffer,byte3(FOUTMAX+1)
		cpc	temp,buffer
		ldd	temp,Z+3
		ldi	buffer,byte4(FOUTMAX+1)
		cpc	temp,buffer
		brcc	a2b_wait3		; FOUT zu gro...
		rjmp	a2b_ok
; ERROR:
A2B_WAIT1:	ldi	temp,1			; ..> Fehler #1
		rcall	error			; ...als Impuls anzeigen
		rjmp	a2b_wait1
;
A2B_WAIT2:	ldi	temp,2			; ..> Fehler #2
		rcall	error			; ...als Impuls anzeigen
		rjmp	a2b_wait2
; 
A2B_WAIT3:	ldi	temp,3			; ..> Fehler #3
		rcall	error			; ...als Impuls anzeigen
		rjmp	siq_wait3
; Copy:
A2B_OK:		ldiw	Z,buf64			; Quelle:
		ldiw	Y,binqrg		; Ziel:
		ldi	counter,4		; 4Bytes = 32Bit
		rcall	movez2y			; ...kopieren
; -------------
		ret



; -----------------------------------------------------------------------------------------------
; (i2canzahl) Bytes aus Datenbuffer zum Si514 schreiben:
; -----------------------------------------------------------------------------------------------

I2COUT:		rcall	i2cstart			; Startbedingung senden
		ldiw	Z,i2cdaten			; Quelle der Daten
		rjmp	i2cadr				; I2C-I/O abarbeiten...
; -------------
; I2CSTART:
; -------------
I2CSTART:	sbi	DDRB,SDA			; SDA wird Ausgang
		sbi	PORTB,SDA			; SDA auf 1 setzen
		sbi	PORTB,SCL			; SCL auf 1 setzen
;
		rcall	dly3us
		rcall	dly3us				; warten
		cbi	PORTB,SDA			; Startflanke 1>0 erzeugen bei SCL=1
		rcall	dly4us				; warten
		cbi	PORTB,SCL			; Taktflanke 1>0
		rcall	dly3us				; warten
		ret
; -------------
; I2CADRESSE:
; -------------
I2CADR:		ldi	temp,SI514W			; I2C-Adresse (Write) )laden
		rcall	i2c_sendbyte			; ...und ausgeben
		rcall	i2c_recack			; ACK empfangen?
		brcs	i2cerr				; nein...
; -------------
; I2CWRITE:
; -------------
I2CWDAT:	ld	temp,Z+				; Daten holen
		rcall	i2c_sendbyte			; ...und Daten ausgeben
		rcall	i2c_recack			; ACK empfangen?
		brcs	i2cerr				; nein...
		decram	i2canzahl			; alle Datenbytes geschrieben?
		brne	i2cwdat				; nein...
; -------------
; I2CSTOP:
; -------------
I2CREADY:	rcall	dly5us				; Warte eine 
		rcall	dly5us				; ...Bitzelle ab
I2CSTOP:	cbi	PORTB,SDA			; SDA=0
		rcall	dly3us				; warten
		sbi	PORTB,SCL			; SCL=1
		rcall	dly3us				; warten
		sbi	PORTB,SDA			; SDA=1
		sbi	DDRB,SDA			; SDA wird Ausgang
		sbr	flags0,(1<<ACK)			; Oszillator hat geantwortet
		clc					; --> kein Error zurck melden
		ret
; -------------
; I2C-Error:
; -------------
I2CERR:		rcall	i2cstop				; Buszyklus beenden
		cbr	flags0,(1<<ACK)			; Oszillator hat nicht geantwortet
		sec					; --> ERROR zurck melden
		ret
; -------------
; Byte senden:
; -------------
I2C_SENDBYTE:	sbi	DDRB,SDA			; SDA wird Ausgang
		push	counter
		ldi	counter,8			; 8 Bits bearbeiten
I2C_SENDNEXT:	cbi	PORTB,SCL			; SCL=0
		lsl	temp				; MSB des Datenbytes ins Carry schieben
		brcc	i2c_sendbit0
I2C_SENDBIT1:	sbi	PORTB,SDA			; SDA=1
		rjmp	i2c_sendcont
;
I2C_SENDBIT0:	cbi	PORTB,SDA			; SDA=0
I2C_SENDCONT:	rcall	dly3us				; warten
		sbi	PORTB,SCL			; SCL=1 (aktive Taktflanke erzeugen)
		rcall	dly3us				; warten
		dec	counter				; alle Bits bearbeitet?
		brne	i2c_sendnext			; nein...
		cbi	PORTB,SCL			; SCL=0
		pop	counter
		ret
; -------------
; Byte empfangen:
; -------------
I2C_RECBYTE:	cbi	DDRB,SDA			; SDA wird Eingang
		push	counter
		ldi	counter,8			; 8 Bits bearbeiten
		clr	temp				; Ergebnisbyte initialisieren
I2C_RECNEXT:	cbi	PORTB,SCL			; SCL=0
		rcall	dly3us				; warten
		sbi	PORTB,SCL			; SCL=1
		rcall	dly2us				; Abtastung etwa etwa in der Mitte des Datenbits
		clc					; Voreinstelung D=0
		sbic	PINB,SDA			; Daten abtasten: SDA=0 ?
		sec					; D=1
		rcall	dly1us				; warten
		rol	temp				; Datenbit im LSB speichern
		dec	counter				; alle Bits bearbeitet?
		brne	i2c_recnext			; nein...
		cbi	PORTB,SCL			; SCL=0
		pop	counter
		ret
; -------------
; ACK lesen:
; -------------
I2C_RECACK:	cbi	DDRB,SDA			; Bus auf "Lesen" setzen
		sbi	PORTB,SDA			; aktiviere PullUp an SDA
		rcall	dly3us				; warten
		sbi	PORTB,SCL			; SCL=1 (aktive Taktphase erzeugen)
		rcall	dly2us				; Abtastung etwa in der Mitte des Datenbits
		sec					; ACK=1 setzen
		sbis	PINB,SDA			; ACK-Bit einlesen
		clc					; Besttigung erhalten...
		pushsr					; Ergebnis retten
		rcall	dly1us				; warten
		cbi	PORTB,SCL			; SCL=0
		rcall	dly3us				; warten
		sbi	PORTB,SDA			; SDA=1
		sbi	DDRB,SDA			; SDA wird wieder Ausgang
		popsr					; ACK in CY zurck geben
		ret
; -------------
; ACK senden:
; -------------
I2C_SENDACK:	sbi	DDRB,SDA			; Bus auf "Schreiben" setzen
		rcall	dly1us				; warten
		cbi	PORTB,SDA			; ACK setzen
		rcall	dly2us				; warten
		sbi	PORTB,SCL			; SCL=1 (aktive Taktphase erzeugen)
		rcall	dly3us				; Abtastung ermglichen
		cbi	PORTB,SCL			; SCL=0
		cbi	DDRB,SDA			; Bus auf "Lesen" setzen
		ret



; -----------------------------------------------------------------------------------------------
; Exklusive Verzgerung 1..5us @ 8 MHz:
; -----------------------------------------------------------------------------------------------

DLY5US:		rcall	dly1us
DLY4US:		rcall	dly1us
DLY3US:		rcall	dly1us
DLY2US:		rcall	dly1us
DLY1US:		rcall	dlyret			; (3) Dauer: 8x125ns = 1s @ 8MHz
DLYRET:		nop				; (1)
		ret				; (4)
